--------------------------------
--	  SET UP	        --
--------------------------------

--ARE YOU A MAGE?--
if select(2, UnitClass("player")) ~= "MAGE" then
	DisableAddOn("Frozen")
	return
end

local Frozen = CreateFrame("Frame")

--LOCALS--
local dur = 8	--Frost Nova duration
local counter = 0	--timer starting var
local throttle = 0	--throttle var for OnUpdate
local frBarBG, frBar, frIcon, frText, frTime	--frames/fontstrings
local LSM = LibStub("LibSharedMedia-3.0")
local widgetLists = AceGUIWidgetLSMlists
local frostnova, db

--DEFAULTS--
local defaults = {
	texture = "Blizzard",
	width = 300,
	height = 20,
	xpos = 0,
	ypos = 0,
	barcolor = {
		r = 0.2,
		g = 0.4,
		b = 0.8,
		a = 1},
	bgcolor = {
		r = 0,
		g = 0,
		b = 0,
		a = .5},
	icon = true,
	font = "Arial Narrow",
	fsize = 14,
}

-------------------------------------
-- MEAT AND POTATOES --
-------------------------------------

local function OnUpdate(self, elapsed)
	throttle = throttle + elapsed
	if throttle >= .05 then
		--MAKE THE COUNTER ACTUALLY COUNT--
		counter = counter + throttle
		throttle = 0
		--UPDATE OUR STATUSBAR--
		frBar:SetValue(dur - counter)
		frTime:SetFormattedText("%.1f",dur - counter)
		--HIDE BAR AND DISABLE TIMER WHEN FINISHED--
		if (dur - counter) <= 0 then
			frBarBG:Hide()
		end
	end
end

local function CreateFrames()
	--THE BACKGROUND FRAME--
	frBarBG = CreateFrame("Frame", nil, UIParent)
	frBarBG:SetPoint("CENTER", UIParent, "CENTER", db.xpos, db.ypos)
	frBarBG:SetWidth(db.width)
	frBarBG:SetHeight(db.height)
	frBarBG:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"})
	frBarBG:SetBackdropColor(db.bgcolor.r, db.bgcolor.g, db.bgcolor.b, db.bgcolor.a)
	frBarBG:SetScript("OnUpdate", OnUpdate)

	--THE STATUSBAR--
	frBar = CreateFrame("StatusBar", nil, frBarBG)
	frBar:SetPoint("CENTER", frBarBG, "CENTER")
	frBar:SetWidth(db.width - 2)
	frBar:SetHeight(db.height - 2)
	frBar:SetStatusBarTexture(LSM:Fetch("statusbar", db.texture))
	frBar:SetStatusBarColor(db.barcolor.r, db.barcolor.g, db.barcolor.b, db.barcolor.a)
	frBar:SetMinMaxValues(0, dur)
	
	--THE ICON--
	frIcon = frBarBG:CreateTexture(nil, "OVERLAY")
	frIcon:SetPoint("RIGHT", frBarBG, "LEFT", -2, 0)
	frIcon:SetWidth(db.height)
	frIcon:SetHeight(db.height)
	frIcon:SetTexture("Interface\\Icons\\Spell_Frost_FrostNova")
	frIcon:SetParent(frBarBG)
	if db.icon == false then frIcon:Hide() end
	
	--THE BAR TEXT--
	frText = frBar:CreateFontString(nil, "OVERLAY")
	frText:SetParent(frBar)
	frText:SetFont(LSM:Fetch("font", db.font), db.fsize)
	frText:SetPoint("LEFT", frBarBG, "LEFT", 5, 0)
	frText:SetText(frostnova)
	
	--THE TIMER TEXT--
	frTime = frBar:CreateFontString(nil, "OVERLAY")
	frTime:SetParent(frBar)
	frTime:SetFont(LSM:Fetch("font", db.font), db.fsize)
	frTime:SetPoint("RIGHT", frBarBG, "RIGHT", -5, 0)
end

local function Frozen_UNIT_SPELLCAST_SUCCEEDED(unit, _, _, _, spellID)
	--DID I CAST FROST NOVA?--
	if unit == "player" and spellID == 122 then
		--CREATE OUR STATUSBAR (why make them, if we haven't used frost nova yet?)
		if not frBarBG then
			CreateFrames()
			CreateFrames = nil
		end
		--START/RESET OUR ONUPDATE TIMER--
		counter = 0
		frBar:SetValue(dur)
		frBarBG:Show()
	end
end

-----------------------------------
-- 	   OPTIONS 	 --
-----------------------------------

local options = {
	name = "Frozen",
	type = "group",
	args = {
		divline = {
			type = "header",
			name = "",
			order = 1,
		},
		xpos = {
			name = "X Position",
			desc = "The horizontal placement of the bar.",
			type = "range",
			min = -1000,
			max = 1000,
			step = 5,
			get = function() return db.xpos end,
			set = function(_, xpos)
					db.xpos = xpos
					frBarBG:ClearAllPoints()
					frBarBG:SetPoint("CENTER", UIParent, "CENTER", xpos, db.ypos)
				end,
			order = 2,
		},
		ypos = {
			name = "Y Position",
			desc = "The vertical placement of the bar.",
			type = "range",
			min = -600,
			max = 600,
			step = 5,
			get = function() return db.ypos end,
			set = function(_, ypos)
					db.ypos = ypos
					frBarBG:ClearAllPoints()
					frBarBG:SetPoint("CENTER", UIParent, "CENTER", db.xpos, ypos)
				end,
			order = 3,
		},
		div1 = {
			name = " ",
			type = "description",
			order = 3.5,
		},
		width = {
			name = "Width",
			desc = "The width of the bar.",
			type = "range",
			min = 50,
			max = 500,
			step = 5,
			get = function() return db.width end,
			set = function(_, wvalue) 
						db.width = wvalue
						frBarBG:SetWidth(wvalue)
						frBar:SetWidth(wvalue - 2)
					end,
			order = 4,
		},
		height = {
			name = "Height",
			desc = "The height of the bar.",
			type = "range",
			min = 5,
			max = 50,
			step = 1,
			get = function() return db.height end,
			set = function(_, hvalue)
						db.height = hvalue
						frBarBG:SetHeight(hvalue)
						frBar:SetHeight(hvalue - 2)
						frIcon:SetHeight(hvalue)
						frIcon:SetWidth(hvalue)
					end,
			order = 5,
		},
		separator1 = {
			type = "header",
			name = "",
			cmdHidden = true,
			order = 6,
		},
		icon = {
			name = "Icon",
			desc = "Toggle showing the icon next to the bar.",
			type = "toggle",
			get = function() return db.icon end,
			set = function()
						db.icon = not db.icon
						if db.icon == false then
							frIcon:Hide()
						else
							frIcon:Show()
						end
					end,
			order = 7,
		},
		texture = {
			name = "Texture",
			desc = "The status bar texture used.",
			type = "select",
			dialogControl = "LSM30_Statusbar",
			values = widgetLists.statusbar,
			get = function()
					return db.texture
				end,
			set = function(_, texture)
					db.texture = texture
					frBar:SetStatusBarTexture(LSM:Fetch("statusbar", texture))
				end,
			order = 8,
		},
		div2 = {
			name = " ",
			type = "description",
			order = 8.5,
		},
		barcolor = {
			name = "Bar Color",
			desc = "The color of the bar.",
			type = "color",
			hasAlpha = true,
			get = function() db = db return db.barcolor.r, db.barcolor.g, db.barcolor.b, db.barcolor.a end,
			set = function(_,r,g,b,a)
					db.barcolor.r = r
					db.barcolor.g = g
					db.barcolor.b = b
					db.barcolor.a = a
					frBar:SetStatusBarColor(r,g,b,a)
				end,
			order = 9,
		},
		bgcolor = {
			name = "Background Color",
			desc = "The color of the bar's background.",
			type = "color",
			hasAlpha = true,
			get = function() db = db return db.bgcolor.r, db.bgcolor.g, db.bgcolor.b, db.bgcolor.a end,
			set = function(_,r,g,b,a)
					db.bgcolor.r = r
					db.bgcolor.g = g
					db.bgcolor.b = b
					db.bgcolor.a = a
					frBarBG:SetBackdropColor(r,g,b,a)
				end,
			order = 10,
		},
		separator2 = {
			type = "header",
			name = "",
			cmdHidden = true,
			order = 11,
		},
		font = {
			name = "Font",
			desc = "The font used for bar text.",
			type = "select",
			dialogControl = "LSM30_Font",
			values = widgetLists.font,
			get = function()
					return db.font
				end,
			set = function(_, font)
					db.font = font
					frText:SetFont(LSM:Fetch("font", font), db.fsize)
					frTime:SetFont(LSM:Fetch("font", font), db.fsize)
				end,
			order = 12,
		},
		fsize = {
			name = "Font size",
			desc = "The size of the font on the bar.",
			type = "range",
			min = 6,
			max = 24,
			step = 1,
			get = function() return db.fsize end,
			set = function(_, fsize)
					db.fsize = fsize
					frText:SetFont(LSM:Fetch("font", db.font), fsize)
					frTime:SetFont(LSM:Fetch("font", db.font), fsize)
				end,
			order = 13,
		},
		separator3 = {
			type = "header",
			name = "",
			cmdHidden = true,
			order = 14,
		},
		test = {
			name = "Test",
			desc = "Shows the bar in action.",
			type = "execute",
			width = "double",
			func = function ()
					if not frBarBG then
						CreateFrames()
					end
					counter = 0
					frBarBG:Show()
					frBar:SetValue(dur)
				end,
			order = 15,
		},
	},
}

local function Frozen_PLAYER_LOGIN()
	--if upgrading, then wipe old saved variables
	if FrozenDB and FrozenDB.profileKeys then
		FrozenDB = nil
		print("|cff99ccffFrozen:|r Saved variables have been reset. Type /frozen or /fr to reconfigure.")
	end
	
	FrozenDB = FrozenDB or {}
	for k,v in pairs(defaults) do
	    if type(FrozenDB[k]) == "nil" then
	        FrozenDB[k] = v
	    end
	end
	db = FrozenDB
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Frozen", options)
	Frozen.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Frozen", "Frozen")
	
	frostnova = GetSpellInfo(122)
	
	SlashCmdList["FROZEN"] = function()
			if not frBarBG then
				CreateFrames()
				frBarBG:Hide()
			end
			InterfaceOptionsFrame_OpenToCategory("Frozen")
		end
	SLASH_FROZEN1 = "/frozen"
	SLASH_FROZEN2 = "/fr"
end

Frozen:SetScript("OnEvent", function(self,event,...)
		if event == "PLAYER_LOGIN" then
			Frozen_PLAYER_LOGIN()
			Frozen:UnregisterEvent("PLAYER_LOGIN")
			Frozen_PLAYER_LOGIN = nil
		else
			Frozen_UNIT_SPELLCAST_SUCCEEDED(...)
		end
	end)

Frozen:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
Frozen:RegisterEvent("PLAYER_LOGIN")

